#ifndef plugin_api_h
#define plugin_api_h

#include "externdef.h"
#include "PLU_types.h"

/* 
   interface for the plugin side:

*/

/* Helper functions for the plugin (accessible from plugin side)
   These are the functions gui_draw is allowed to call!
   (well, right now it is only one function, so that we can
    make old plugins work, but why not add some more? ;-)
 */

LIBEXPORT void plugin_gui_draw_slider(plugin_control_t * control,
				      int type, const char* name,
				      float def, float min, float max,
				      const char* tip, 
				      float * data, int xpos, int ypos);

LIBEXPORT void plugin_gui_bind_ipo(plugin_control_t * control, 
				   const char * icuname,
				   float * data);
LIBEXPORT void plugin_gui_icu_set_range(plugin_control_t * control, 
					const char * icuname,
					float ymin, float ymax);

LIBEXPORT float plugin_gui_get_ipo_value(plugin_control_t * control, 
					 const char * icuname,
					 float frame);

/* functions to bind instance data to properties 
   these instance elements will be exposed to the core and are stored
   persistently in blend file.

   plugin_bind_properties_varstruct is a convenience function
   for usage with varstructs

   plugin_get / set_property: use this to get / set properties without 
   binding.

*/

typedef struct VarStruct {
	int type;
	char name[16];
	float def, min, max;
	char tip[80];
} VarStruct;

typedef enum {
	PLP_END = -1,
	PLP_STRING,
	PLP_INT,
	PLP_FLOAT
} plp_type_t;

LIBEXPORT int plugin_bind_property(
	plugin_control_t * control,
	plp_type_t prop_type, const char * name, void * data);

LIBEXPORT int plugin_bind_properties(
	plugin_control_t * control,
	plp_type_t prop_type, const char * name, void * data, ...);

LIBEXPORT int plugin_bind_properties_varstruct(
	plugin_control_t * control,
	VarStruct * vstruct,
	unsigned int num_elems, 
	void * begin_of_data);

/* returns false if property not available or type doesn't match! */
LIBEXPORT int plugin_get_property(plugin_control_t * control,
				  plp_type_t prop_type, const char * name, 
				  void * data);
LIBEXPORT int plugin_set_property(plugin_control_t * control,
				  plp_type_t prop_type, const char * name, 
				  void * data);

/* sequencer inputs are able to seek on input, so here we go: 
   (will return null, if no such imbuf exists) 

   IMPORTANT! If you want to return an ImBuf fetched with get_imbuf
   without change, you have to call refImBuf() to increment it's reference
   count!
*/

LIBEXPORT struct ImBuf * plugin_sequencer_get_imbuf(
	plugin_control_t * control, int num_input, float frame);

LIBEXPORT void plugin_sequencer_get_audio(
	plugin_control_t * control, int num_input, 
	float * output, int num_samples);

LIBEXPORT void plugin_audio_to_char(float * in, char * out, int num_samples);
LIBEXPORT void plugin_audio_to_short(float * in, short * out, int num_samples);
LIBEXPORT void plugin_audio_to_int(float * in, int * out, int num_samples);
LIBEXPORT void plugin_audio_to_24bit(float * in, void * out, int num_samples);

/* convenience function to create a new output imbuf from properties */
LIBEXPORT struct ImBuf * plugin_new_imbuf(plugin_control_t * control);



/* 
   interface for the blender core:

   plugin_enumerate: get all descriptors for a certain type by doing
           a refresh of the plugin paths if not done. On first run, we scan
	   all available plugins for there interface and store this in
	   a cache file. (In case you wonder, install gimp and 
           watch startup...)

   plugin_get_by_name: find descriptor by name and type.

   plugin_init_control: load plugin into memory if not done 
           and setup control structures for first use thereby creating 
           a plugin_control structure.

   plugin_load_control: load plugin into memory if not done,
           but reuses plugin_control
           structure, that is already present from file restore. 
	   Does it only on first call, so call as often you like.
	   Check return value! 1 = OK, 0 = load failed! (plugin not available)
      
*/

LIBEXPORT plugin_descriptor_t * plugin_enumerate(
	plugin_type_t type, int force_refresh);
LIBEXPORT plugin_descriptor_t * plugin_get_by_name(
	plugin_type_t type, const char* name);

LIBEXPORT plugin_control_t * plugin_init_control(
	plugin_descriptor_t * d, 
	plugin_uplink_handler_t * uplink_handler,
	void * uplink, 
	int prop_type, const char * name, void * data, ...);

LIBEXPORT int plugin_load_control(plugin_control_t * c, 
				  plugin_uplink_handler_t * uplink_handler,
				  void * uplink);
LIBEXPORT void plugin_free_control(plugin_control_t * c);

/*
  File open using this interface might look like this:

  --------------
  plugin_control_t * open_video_file(const char * filename) {
        plugin_control_t * rv;

	plugin_descriptor_t * p = plugin_enumerate(PLUGIN_TYPE_VIDEO_INPUT);
  
	while (p) {
	     if (file_matches_pattern(filename, p->file_pattern)) {
	           rv = plugin_init_control(p, NULL, NULL,
	                                    PLP_STRING, "filename", filename,
					    PLP_END);
		   if (rv) {
	                return rv;
		   }
             }

             p = p->next;
        }

        return 0;
  }

  --------------
  video plugin looks like this:
  --------------

  #include "plugin.h"

  struct plugin_instance {
        int xoffset;
	int yoffset;

	int crop_width;
	int crop_height;
	
	jpeghandle_t handle;
  };
  
  plugin_instance_t* my_init(plugin_control_t * c) {
        plugin_instance_t * rv = 0;
	char* filename;
	jpeg_handle_t handle = 0;

	if (!plugin_get_property(c, PLP_STRING, "filename", &filename)) {
	      plugin_set_property(c, PLP_STRING, "error", "core went nuts");
     	      return 0;
	}

	... try jpeg open, that returns handle ...

	if (!handle) {
	      plugin_set_property(c, PLP_STRING, "error", failure_notice);
	      return 0;
	}

	rv = mallocN(sizeof(struct plugin_instance), "jpeg-instance");

	plugin_bind_properties(c, 
	                       PLP_INT, "xoffset", &rv->xoffset,
			       PLP_INT, "yoffset", &rv->yoffset,
			       PLP_INT, "width", &rv->crop_width,
			       PLP_INT, "height", &rv->crop_height,
			       PLP_END);
	
	rv->handle = handle;

	return rv;
  }

  void my_free(plugin_instance_t* This) {
        jpeglib_free_handle(This->handle);
  }
	
  struct ImBuf * my_get(plugin_control_t* c) {
        struct ImBuf * rv = plugin_new_imbuf(c);

	unsigned int * rect = imbuf_get_rect(rv);
	
	... jpeg load ...


  }

  void plugin_query() {
        plugin_register_video_input("jpeg", 
	                            "Author",
				    "Copyright (C) 2007 Author",
				    "Opens JPEG files using jpeglib",
				    "Today ;-)",
				    "*.jpg;*.jpeg",
	
				    PLUGIN_CAP_IMBUF_BYTE,
	
				    0,

				    my_init, my_init, my_free,
			    
				    doit);
   }

*/
  

/* 
   interface for the plugin to the blender core
   
   registration

   NOTE! Use functions from PLU_register.h 
   _never_ _ever_ plugin_register_private!
*/

LIBEXPORT plugin_descriptor_t * plugin_register_private(
	int interface_version,
	plugin_type_t type,
	unsigned long capabilities,
	const char* name,
	const char* menu_entry,
	const char* author,
	const char* copyright,
	const char* help,
	const char* date,
	const char* file_pattern,
	plugin_private_descriptor_t * desc,
	plugin_instance_t* (*init)(plugin_control_t * c),
	plugin_instance_t* (*load)(plugin_control_t * c),
	void (*free)(plugin_instance_t* This));

#include "PLU_register.h"

/* register custom gui draw() and callback() functions for special
   n-keys dialog
*/

/* 3 alternative ways to register a gui.
   If you don't understand this, use the last option... ;-) 
*/

LIBEXPORT void plugin_register_gui(
	plugin_descriptor_t * desc,

	void (*draw)(plugin_control_t * control),
	void (*callback)(plugin_control_t * control, short button));


LIBEXPORT void plugin_register_gui_custom_init(
	plugin_descriptor_t * desc,

	void (*init)(plugin_control_t * control),
	void (*load)(plugin_control_t * control),
	void (*free)(plugin_control_t * control),

	void (*draw)(plugin_control_t * control),
	void (*callback)(plugin_control_t * control, short button));


LIBEXPORT void plugin_register_gui_simple(
	plugin_descriptor_t * desc,

	VarStruct * vstruct,
	unsigned int num_elems,
	unsigned int offset_p_data /* offset into plugin_instance_t,
				      where we should store the
				      persitent data array
				      (formerly known as Cast)
				   */
	); 




/* texture helper functions */

/* derived from the famous Perlin noise */
LIBEXPORT float hnoise(float noisesize, float x, float y, float z);
/* the original Perlin noise */
LIBEXPORT float hnoisep(float noisesize, float x, float y, float z);

/* soft turbulence */
LIBEXPORT float turbulence(float noisesize, float x, float y, float z, int depth);
/* hard turbulence */
LIBEXPORT float turbulence1(float noisesize, float x, float y, float z, int depth);



#endif
