#ifndef plugin_types_h
#define plugin_types_h

/*
  PLUGIN_TYPE: the different plugin types, that can register itself.
  SUB_TYPEs aren't real types, but are used instead as a convenience for
  plugin authors. A point filter for instance is propagated to a
  compositor node and a sequencer plugin at once.
*/

typedef enum PLUGIN_TYPE {
	PLUGIN_TYPE_VIDEO_INPUT = 0,
	PLUGIN_TYPE_VIDEO_EFFECT,
	PLUGIN_TYPE_VIDEO_OUTPUT,

	PLUGIN_TYPE_VIDEO_INPUT_DEVICE,
	PLUGIN_TYPE_VIDEO_OUTPUT_DEVICE,

	PLUGIN_TYPE_AUDIO_INPUT,
	PLUGIN_TYPE_AUDIO_EFFECT,
	PLUGIN_TYPE_AUDIO_OUTPUT,

	PLUGIN_TYPE_AUDIO_RESAMPLER,

	PLUGIN_TYPE_AUDIO_INPUT_DEVICE,
	PLUGIN_TYPE_AUDIO_OUTPUT_DEVICE,

	PLUGIN_TYPE_SUBTITLE_INPUT,
	PLUGIN_TYPE_SUBTITLE_EFFECT, 
	PLUGIN_TYPE_SUBTITLE_OUTPUT,

	PLUGIN_TYPE_MUX_OUTPUT,

	PLUGIN_TYPE_MATERIAL,

	PLUGIN_TYPE_COMPOSITOR_NODE,
	PLUGIN_TYPE_MATERIAL_NODE,

	PLUGIN_SUB_TYPE_VIDEO_POINT_FILTER,
	
	PLUGIN_SIZEOF_TYPE /* _always_ the last element */
} plugin_type_t;

/*
  plugin_instance: temporary private data, that is reconstructed on
  load / plugin construction / refresh
  replaces: struct anim, struct hdaudio, ...

  Noone besides plugin implementors should care about... ;-)
*/

struct plugin_instance;
typedef struct plugin_instance plugin_instance_t;

/*
  plugin_descriptor: same intention as old plugin_info, but redesigned and
  therefor renamed to prevent name clash.

  Every plugin registers one or more plugins using a plugin_descriptor.
  This is the first extension to the old system: not one plugin per file,
  but possibly more than one. This is necessary to provide all subplugins
  for gstreamer for example.

  Each descriptor also holds a private data pointer, that is passed to
  each plugin instance factory. This is necessary to use the same factory
  for several different plugins. Again: gstreamer, but also encapsulation
  for old plugins for example.
*/

struct plugin_private_descriptor;
typedef struct plugin_private_descriptor plugin_private_descriptor_t;

/*
  bMuxPacket: container for a video/audio frame. Used for MUX_OUTPUT only 
    right now. (Could be used also for input, but right now the demuxer is
    included within the input-plugins.)
*/

typedef struct bMuxPacket {
	char mime_type[64];
	long long timestamp; /* in nanoseconds */
	long long duration;  /* in nanoseconds */

	int packet_size;
	int is_key;

	void * data;
} bMuxPacket;

/*
  plugin_control: replaces "PluginSeq" (and similar in material)
  Since instance is only temporary for each session, we need a place
  to store persistent information, enough to recover
  the instance from blend file. 

  plugin_uplink_handler: implementation structure for GUI / Ipos etc.
  implementation is plugin-type-dependent.
  Don't expect anything from these structures as a plugin author. They are
  abstract for you!
*/

struct plugin_control;
typedef struct plugin_control plugin_control_t;

struct plugin_uplink_handler;
typedef struct plugin_uplink_handler plugin_uplink_handler_t;

/*
  plugin_interface_common:

  since all initialisation functions are shared between plugins,
  init / load / free are shared between plugins

  init: called on first initialisation of plugin
  load: called on blend file restore (init is not called !)

  init may return null, if plugin can't be initialized (file format not
  unterstood etc.)

  On simple plugin you can register the same function for init and load!

*/

typedef struct plugin_interface_common {
	plugin_instance_t* (*init)(plugin_control_t * control);
	plugin_instance_t* (*load)(plugin_control_t * control);
	void (*free)(plugin_instance_t* This);
} plugin_interface_common_t;

/*
  plugin_interface_gui:

  init: called on first initialisation of plugin
  load: called on blend file restore (init is not called !)

  On simple plugin you can register the same function for init and load!
  (or as well don't use it at all. Maybe only usefull for generic guis
  used on several different plugin types)
*/

typedef struct plugin_interface_gui {
	void (*init)(plugin_control_t * control);
	void (*load)(plugin_control_t * control);
	void (*free)(plugin_control_t * control);

	void (*draw)(plugin_control_t * control);
	void (*callback)(plugin_control_t * control, short button);
} plugin_interface_gui_t;


/* *********** */
/* The interfaces for the several plugin types follow */

/* also used for image input */

typedef struct plugin_interface_video_input {
	/*
	  the following properties are always set by the core

	  "filename"    filename to open (string)

	  "xoffset"     crop xoffset (int)
	  "yoffset"     crop yoffset (int)
	  "width"       crop width (int)
	  "height"      crop height (int)

	  the following properties are used by the core, if exposed

	  "frame"       frame returned by next call to get() (float)
	  "duration"    duration in frames (float)
	  "framerate"   framerate (float)
	*/

	struct ImBuf * (*get)(plugin_control_t* c);

} plugin_interface_video_input_t;

typedef struct plugin_interface_video_effect {
	/*
	  the following properties are always set by the core

	  "frame"       frame returned by next call to process() (float)
	  "width"       output width (int)
	  "height"      output height (int)
	  "is_float"    float output (boolean) (only set, if capability ok)

	  the following properties are used by the core, if exposed

	  "numinputs"   number of maximum inputs supported by effect
	*/
	
	struct ImBuf * (*process)(plugin_control_t* c);
} plugin_interface_video_effect_t;


typedef struct plugin_interface_video_output {
	/*
	  the following properties are always set by the core

	  "frame"         current frame in append()
	  "framerate"     frame rate
	  "width"         output width
	  "height"        output height
	  "is_float"      float output (boolean) (only set, if capability ok)
	  "is_multilayer" output is multilayer   (only set, if capability ok)

	  the following properties are used by the core, if exposed

	  none yet.

	*/

	void (*process)(plugin_control_t * c, 
			struct ImBuf * in, bMuxPacket * out);

} plugin_interface_video_output_t;


typedef struct plugin_interface_audio_input {
	/*
	  the following properties are always set by the core

	  "filename"    current filename
	  "samplepos"   sample_position
	  "numsamples"  number of samples

	  the following properties are used by the core, if exposed

	  "duration"    duration in samples (mandatory)
	  "numchannels" number of channels  (mandatory)
	  "samplerate"  sample rate         (mandatory)
	*/


	/* prebuffering is done in the core in n seperate threads 

 	   automatic resampling is also done in the core.
	   (this is a change in interface regarding hddaudio here...)
	*/

	void (*get)(plugin_control_t* c, float * target_buffer);
} plugin_interface_audio_input_t;

typedef struct plugin_interface_audio_effect {
	/*
	  the following properties are always set by the core

	  "samplepos"   sample_position
	  "numchannels" number of channels in mixer
	  "samplerate"  sample rate
	  "numsamples"  number of samples to process

	  the following properties are used by the core, if exposed

	  none yet.

	*/

	void (*process)(plugin_control_t* c, float * target_buffer);

} plugin_interface_audio_effect_t;

typedef struct plugin_interface_audio_output {
	/*
	  the following properties are always set by the core

	  "samplepos"   sample_position
	  "numchannels" number of channels in mixer
	  "samplerate"  sample rate
	  "numsamples"  number of samples to process

	  the following properties are used by the core, if exposed
	  (mix_buffer is automatically resampled to match output, if needed)

	  "outchannels"   number of output channels
	  "outsamplerate" sample rate in output

	*/

	void (*process)(plugin_control_t * c, float * source_buffer, 
			bMuxPacket * out);

} plugin_interface_audio_output_t;

typedef struct plugin_interface_audio_resampler {
	/*
	  the following properties are always set by the core

	  "samplepos"   sample_position
	  "numchannels" number of channels in mixer
	  "samplerate"  sample rate
	  "numsamples"  number of samples to process (in output)
	  "outchannels"   number of output channels
	  "outsamplerate" sample rate in output

	  the following properties are used by the core, if exposed
	  
	  none yet

	*/

	/* returns number of samples processed in input */

	int (*process)(plugin_control_t* c, 
		       float * source_buffer, float * target_buffer);

} plugin_interface_audio_resampler_t;

typedef struct plugin_interface_audio_output_device {
	/*
	  the following properties are always set by the core

	  "samplepos"   sample_position
	  "numchannels" number of channels in mixer
	  "samplerate"  sample rate
	  "numsamples"  number of samples to process

	  the following properties are used by the core, if exposed
	  (mix_buffer is automatically resampled to match output, if needed)

	  "outchannels"   number of output channels
	  "outsamplerate" sample rate in output

	*/

	void (*process)(plugin_control_t * c, float * source_buffer);

} plugin_interface_audio_output_device_t;


typedef struct plugin_interface_mux_output {
	/*
	  the following properties are always set by the core

	  "filename"      current filename
	  "format_type"   format mime type
	  "video_type"    video mime type
	  "audio_type"    audio mime type
	  "subtitle_type" subtitle mime type
	  "framerate"     frame rate
	  "width"         output width
	  "height"        output height
	  "is_float"      float output (boolean) (only set, if capability ok)
	  "is_multilayer" output is multilayer   (only set, if capability ok)

	  the following properties are used by the core, if exposed

	  "error"         error string 
	                  (set this, to stop rendering with a message)
	*/

	/* start / end map to init / free, since we need an instance only
	   for the duration of one rendering */

	int (*want_audio)(plugin_control_t * c, 
			  int * sample_pos, int * numsamples);
	int (*want_video)(plugin_control_t * c, 
			  int * next_frame);

	void (*process)(plugin_control_t * c, bMuxPacket * pkt);

} plugin_interface_mux_output_t;


typedef struct plugin_interface_video_point_filter {
	/*
	  the following properties are always set by the core

	  none yet

	  the following properties are used by the core, if exposed

	  none yet

	*/
	
	void (*process)(plugin_control_t * c, float * in_buffer,
			float * out_buffer, int n_pixels);
} plugin_interface_video_point_filter_t;

typedef struct plugin_interface_material {
	/*
	  materials don't use any properties by default for speed reasons,
	  but can add some for themselfs, if they like.
	*/
	
	float * (*process)(plugin_control_t* c,
			   float cfra,
			   int stype, float *texvec, float *dxt, float *dyt );
} plugin_interface_material_t;

/*
  plugin descriptor: read from a seperate plugin registry and / or by file
  scan. Describes the plugin and offers the interface to create an instance.

  Here is also the right place to check for capabilities.
  (in the old version, we mixed up capabilities with versioning.
  version means: my interface conforms to structure version x.
  where as capability means: hey, I can do float imbufs, but I can't do
  byte, sorry!)

*/

#define	PLUGIN_CAP_IMBUF_FLOAT       (1 << 0)
#define	PLUGIN_CAP_IMBUF_BYTE        (1 << 1)
#define	PLUGIN_CAP_IMBUF_MULTILAYER  (1 << 2)

typedef struct plugin_descriptor {
	struct plugin_descriptor *next, *prev;

	plugin_type_t plugin_type;

	unsigned long capabilities;

	char* name;
	char* plugin_filename;
	char* menu_entry;

	char* author;
	char* copyright;
	char* help;
	char* date;

	char* file_pattern;

	time_t lastchanged;

	/* fields below this place are initialized by plugin_load! */

	int interface_version; /* =0 if interface not loaded ! */
	
	plugin_private_descriptor_t * desc;

	plugin_interface_common_t common;
	plugin_interface_gui_t * gui;

	union {
		plugin_interface_video_input_t video_in;
		plugin_interface_video_effect_t video_eff;
		plugin_interface_video_output_t video_out;

		plugin_interface_audio_input_t audio_in;
		plugin_interface_audio_effect_t audio_eff;
		plugin_interface_audio_output_t audio_out;

		plugin_interface_audio_resampler_t audio_res;
		plugin_interface_audio_output_device_t audio_out_dev;

		/*
		plugin_interface_subtitle_input_t subtitle_in;
		plugin_interface_subtitle_effect_t subtitle_eff;
		plugin_interface_subtitle_output_t subtitle_out;
		*/

		plugin_interface_mux_output_t mux_output;
		plugin_interface_material_t material;
		/*
		plugin_interface_compositor_node_t compositor_node;
		plugin_interface_material_node_t material_node;
		*/

		plugin_interface_video_point_filter_t point_filter;

		/* add more as need arises ... ;-) */
	} i;
} plugin_descriptor_t;


#endif
